<?xml version="1.0" encoding="utf-8"?>
<html xmlns="http://www.w3.org/1999/xhtml" version="XHTML+RDFa 1.0">
  <head>
    <title>Bogus ASP.NET test</title>
  </head>
  <body>
    <p>here i am</p>
  </body>
</html>
